# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https: // mozilla.org / MPL / 2.0 / .
# Copyright 2021 Peter Dunne
"""Fit data for magnetic field models



Todo:
    * Reimplement all 
"""
pass

# def fit_cube(z, Jr, dz):
#     xDim = 10/2000
#     yDim = 10/2000
#     zDim = 10/2000
#     return(mag.Bzz1d(xDim, yDim, zDim, Jr, z-dz))
#     # return(mag.solBz(xDim,yDim,yDim + y-dy,n,I))


# def fit_cyl(y, Jr, dy):
#     R = 14/2000
#     HH = 10/2000
#     n = 1
#     I = Jr/(u0*n)
#     # return(mag.Byy2d(xDim, yDim, Jr, 0, yDim + y-dy))
#     return(mag.solBz(R, HH, HH+y-dy, n, I))
